@@ -1,20 +1,26 @@
 # -*- coding: utf-8 -*-
 """k-Nearest Neighbors Detector (kNN)
 """
-# Author: Yue Zhao <yuezhao@cs.toronto.edu>
+# Author: Yue Zhao <zhaoy@cmu.edu>
 # License: BSD 2 clause
 from __future__ import division
 from __future__ import print_function
 
+from warnings import warn
+
 import numpy as np
 from sklearn.neighbors import NearestNeighbors
-from sklearn.neighbors import KDTree
+from sklearn.neighbors import BallTree
 from sklearn.utils import check_array
 from sklearn.utils.validation import check_is_fitted
 
 from .base import BaseDetector
 
 
+# TODO: algorithm parameter is deprecated and will be removed in 0.7.6.
+# Warning has been turned on.
+# TODO: since Ball_tree is used by default, may introduce its parameters.
+
 class KNN(BaseDetector):
     # noinspection PyPep8
     """kNN class for outlier detection.
@@ -62,8 +68,12 @@ class KNN(BaseDetector):
         Note: fitting on sparse input will override the setting of
         this parameter, using brute force.
 
+        .. deprecated:: 0.74
+           ``algorithm`` is deprecated in PyOD 0.7.4 and will not be
+           possible in 0.7.6. It has to use BallTree for consistency.
+
     leaf_size : int, optional (default = 30)
-        Leaf size passed to BallTree or KDTree.  This can affect the
+        Leaf size passed to BallTree. This can affect the
         speed of the construction and query, as well as the memory
         required to store the tree.  The optimal value depends on the
         nature of the problem.
@@ -144,6 +154,11 @@ class KNN(BaseDetector):
         self.metric_params = metric_params
         self.n_jobs = n_jobs
 
+        if self.algorithm != 'auto' and self.algorithm != 'ball_tree':
+            warn('algorithm parameter is deprecated and will be removed '
+                 'in version 0.7.6. By default, ball_tree will be used.',
+                 FutureWarning)
+
         self.neigh_ = NearestNeighbors(n_neighbors=self.n_neighbors,
                                        radius=self.radius,
                                        algorithm=self.algorithm,
@@ -155,24 +170,43 @@ class KNN(BaseDetector):
                                        **kwargs)
 
     def fit(self, X, y=None):
-        """Fit detector. y is optional for unsupervised methods.
+        """Fit detector. y is ignored in unsupervised methods.
 
         Parameters
         ----------
         X : numpy array of shape (n_samples, n_features)
             The input samples.
 
-        y : numpy array of shape (n_samples,), optional (default=None)
-            The ground truth of the input samples (labels).
+        y : Ignored
+            Not used, present for API consistency by convention.
+
+        Returns
+        -------
+        self : object
+            Fitted estimator.
         """
 
         # validate inputs X and y (optional)
         X = check_array(X)
         self._set_n_classes(y)
 
-        self.tree_ = KDTree(X, leaf_size=self.leaf_size, metric=self.metric)
         self.neigh_.fit(X)
 
+        # In certain cases, _tree does not exist for NearestNeighbors
+        # See Issue #158 (https://github.com/yzhao062/pyod/issues/158)
+        # n_neighbors = 100
+        if self.neigh_._tree is not None:
+            self.tree_ = self.neigh_._tree
+
+        else:
+            if self.metric_params is not None:
+                self.tree_ = BallTree(X, leaf_size=self.leaf_size,
+                                      metric=self.metric,
+                                      **self.metric_params)
+            else:
+                self.tree_ = BallTree(X, leaf_size=self.leaf_size,
+                                      metric=self.metric)
+
         dist_arr, _ = self.neigh_.kneighbors(n_neighbors=self.n_neighbors,
                                              return_distance=True)
         dist = self._get_dist_by_method(dist_arr)
